import React, { useState, useEffect } from 'react'
import { Input, Divider, Loader, Segment, Card, Dropdown, Button, Form, TextArea } from 'semantic-ui-react'
import { useSelector, useDispatch } from 'react-redux'
import { Link, useParams, useNavigate } from 'react-router-dom'
import { toast } from 'react-toastify'
import actions from '../../../actions'
import api from '../../../api'
import utils from '../../../utils/utils'
import UserChip from '../../includes/UserChip'

export default function ForumTopic () {
  const [newReply, setNewReply] = useState('')
  const [totalReplies, setTotalReplies] = useState(0)
  const [totalPages, setTotalPages] = useState(1)
  const [topics, setTopics] = useState([])
  const [loadingReplies, setLoadingReplies] = useState(false)
  const dispatch = useDispatch()
  const navigate = useNavigate()
  const { id, topicId, page : pageParam } = useParams()
  const page = parseInt(pageParam) || 1

  const { user, group, topic, replies } = useSelector(state => {
    const group = state.groups.groups.filter(g => g._id === id)[0]
    const user = state.users.users.filter(u => state.auth.currentUserId === u._id)[0]
    const topic = state.groups.topics.filter(t => t._id === topicId)[0]
    const replies = state.groups.topicReplies.filter(r => r.topic === topicId)
    return { user, group, topic, replies }
  })

  useEffect(() => {
    setLoadingReplies(true)
    api.groups.getForumTopicReplies(group._id, topicId, page, ({ topic, replies, totalReplies, totalPages }) => {
      setLoadingReplies(false)
      dispatch(actions.groups.receiveTopics([topic]))
      dispatch(actions.groups.receiveTopicReplies(replies))
      setTotalReplies(totalReplies)
      setTotalPages(totalPages)
    }, err => {
      toast.error(err.message)
      setLoadingReplies(false)
    })
  }, [group._id, topicId, page])

  function handleReply () {
    if (!newReply) return
    api.groups.createForumTopicReply(group._id, topicId, { content: newReply }, reply => {
      setNewReply('')
      dispatch(actions.groups.receiveTopicReplies([reply]))
    }, err => {
      toast.error(err.message)
    })
  }

  const BASE_ROUTE = `/groups/${id}/forum/topics/${topicId}`

  return (
    <div>
      <Button basic as={Link} to={`/groups/${id}/forum`}>View all topics</Button>
      <h2>{topic?.title || 'Topic'}</h2>

      {loadingReplies && <Loader active />}
      {!loadingReplies && replies.length === 0 && <p>No replies yet</p>}

      {replies.map(reply => (
        <Segment key={reply._id}>
          <div style={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center' }}>
            <UserChip user={reply.author} compact />
            <div style={{ fontSize: 12, color: 'rgb(150,150,150)' }}>{new Date(reply.createdAt).toLocaleString()}</div>
          </div>
          <p>{reply.content}</p>
        </Segment>
      ))}

      {totalPages > 1 &&
        <div style={{ display: 'flex', justifyContent: 'center', alignItems: 'center', marginTop: 20, marginBottom: 20 }}>
          <Button size='tiny' disabled={page === 1} as={Link} to={`${BASE_ROUTE}/page/${page - 1}`}>Previous page</Button>
          <span style={{ margin: '0 10px' }}>Page {page} of {totalPages}</span>
          <Button size='tiny' disabled={page === totalPages} as={Link} to={`${BASE_ROUTE}/page/${page + 1}`}>Next page</Button>
        </div>
      }

      {utils.hasGroupPermission(user, group, 'postForumTopicReplies') && page === totalPages &&
        <Form>
          <TextArea placeholder='Write a reply to this topic...' value={newReply} onChange={e => setNewReply(e.target.value)} />
          <Button color='teal' onClick={handleReply} style={{ marginTop: 10 }}>Reply</Button>
        </Form>}

    </div>
  )
}
